@@ -26,6 +26,7 @@ from ..utils.compiler import Compiler
 from ..tasks.task import TaskStatus
 from ..tasks.dummy import DummyTask
 from ..tasks.sql import SqlTask
+
 from ..tasks.autosql import AutoSqlTask
 from ..tasks.copy import CopyTask
 from ..tasks.test import TestTask
@@ -55,7 +56,7 @@ class RunArguments:
         sql: str = str(Path("sql"))
         compile: str = str(Path("compile"))
         logs: str = str(Path("logs"))
-        tests: str = str(Path("sql") / Path("tests"))
+        tests: str = str(Path("sql"))
 
     folders: Folders
     full_load: bool = False
@@ -179,15 +180,6 @@ class App:
 
         self.check_abort(self.set_tasks(tasks_dict))
 
-        # Apply the task query
-        self.task_query = self.check_abort(
-            get_query(
-                tasks_dict,
-                include=self.run_arguments.include,
-                exclude=self.run_arguments.exclude,
-            )
-        )
-
         self.tracker.finish_current_stage(
             tasks={k: v.status for k, v in self.tasks.items()},
             test=True if self.run_arguments.command == Command.TEST else False,
@@ -435,16 +427,9 @@ class App:
             if result.is_err:
                 return result
 
-        if self.run_arguments.command == Command.TEST:
-            # TODO move this logic to the task config
-            self.dag = {
-                task.name: [] for task in task_objects.values() if task.has_tests()
-            }
-        else:
-            self.dag = {
-                task.name: [p.name for p in task.parents]
-                for task in task_objects.values()
-            }
+        self.dag = {
+            task.name: [p.name for p in task.parents] for task in task_objects.values()
+        }
 
         topo_sort = topological_sort(self.dag)
         if topo_sort.is_err:
@@ -457,18 +442,35 @@ class App:
         return Ok()
 
     def setup_execution(self):
+        # Apply the task query
+        tasks_dict = {
+            name: {"group": task.group, "tags": list(task.tags)}
+            for name, task in self.tasks.items()
+        }
+        self.task_query = self.check_abort(
+            get_query(
+                tasks_dict,
+                include=self.run_arguments.include,
+                exclude=self.run_arguments.exclude,
+            )
+        )
+
         result = dag_query(self.dag, self.task_query)
         if result.is_err:
             return result
         else:
             tasks_in_query = result.value
 
+        if self.run_arguments.command == Command.TEST:
+            tasks_in_query = [t for t in tasks_in_query if self.tasks[t].has_tests()]
+
         # Introspection
         #########
 
         # Create the list of objects to be used in this execution
         exec_outputs = set()
         exec_sources = set()
+        exec_connections = set()
         for task_name in tasks_in_query:
             for output in self.tasks[task_name].outputs:
                 exec_outputs.add(output)
@@ -482,10 +484,8 @@ class App:
                 #     objects_used[source.connection] = set()
                 # objects_used[source.connection].add(source)
 
-        # We need to introspect the connections used
-        exec_connections = set()
-        exec_connections.update([o.connection_name for o in exec_outputs])
-        exec_connections.update([o.connection_name for o in exec_sources])
+            for connection in self.tasks[task_name].used_connections:
+                exec_connections.add(connection)
 
         # Now that we have done the config for all tasks and we know which
         # connections are required, check that we have them all
@@ -539,11 +539,11 @@ class App:
                     db._activate_connection()  # This call creates the engine and tests the connection
                 except Exception as exc:
                     return Exc(exc, where="create_connection")
-
-                try:
-                    db._introspect(to_introspect[connection_name])
-                except Exception as exc:
-                    return Exc(exc, where="introspection")
+                if connection_name in to_introspect:
+                    try:
+                        db._introspect(to_introspect[connection_name])
+                    except Exception as exc:
+                        return Exc(exc, where="introspection")
 
         self.tracker.set_tasks(tasks_in_query)
 
