@@ -37,33 +37,10 @@ class Columns(BaseModel):
         extra = Extra.forbid
 
 
-class DDL(BaseModel):
-    columns: List[Union[str, Columns]] = list()
-    post_hook: List[Hook] = list()
-
-    class Config:
-        extra = Extra.forbid
-
-    @validator("columns", pre=True)
-    def transform_str_cols(cls, v):
-        if v is not None and isinstance(v, List):
-            return [{"name": c} if isinstance(c, str) else c for c in v]
-        else:
-            return v
-
-    @validator("columns")
-    def columns_unique(cls, v):
-        dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
-        if len(dupes) > 0:
-            raise ValueError(f"Duplicate columns: {','.join(dupes)}")
-        else:
-            return v
-
-    def get_ddl(self):
-        cols = self.columns
-
+class BaseDDL(BaseModel):
+    def base_ddl(self):
         columns = list()
-        for c in cols:
+        for c in self.columns:
             tests = list()
             for t in c.tests:
                 if isinstance(t, str):
@@ -95,6 +72,32 @@ class DDL(BaseModel):
         }
 
 
+class DDL(BaseDDL):
+    columns: List[Union[str, Columns]] = list()
+    post_hook: List[Hook] = list()
+
+    class Config:
+        extra = Extra.forbid
+
+    @validator("columns", pre=True)
+    def transform_str_cols(cls, v):
+        if v is not None and isinstance(v, List):
+            return [{"name": c} if isinstance(c, str) else c for c in v]
+        else:
+            return v
+
+    @validator("columns")
+    def columns_unique(cls, v):
+        dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
+        if len(dupes) > 0:
+            raise ValueError(f"Duplicate columns: {','.join(dupes)}")
+        else:
+            return v
+
+    def get_ddl(self):
+        return self.base_ddl()
+
+
 class Database:
     """
     Base class for databases in SAYN.
@@ -182,7 +185,7 @@ class Database:
             # Force a query to test the connection
             self.execute("select 1")
 
-    def _construct_tests(self, columns, table, schema=None):
+    def _construct_tests_template(self, columns, table, test_file_name, schema):
         query = """
                    SELECT val
                         , col
@@ -190,10 +193,9 @@ class Database:
                         , type
                      FROM (
                 """
-        template = self._jinja_test.get_template("standard_tests.sql")
+        template = self._jinja_test.get_template(test_file_name)
         count_tests = 0
         breakdown = []
-
         for col in columns:
             tests = col["tests"]
             for t in tests:
@@ -204,6 +206,9 @@ class Database:
                         else col["dst_name"],
                         "type": t["type"],
                         "execute": t["execute"],
+                        "allowed_values": ", ".join(
+                            f"'{c}'" for c in t["allowed_values"]
+                        ),
                     }
                 )
 
@@ -229,6 +234,12 @@ class Database:
             query += q.strip() + "\n"
         query += ") AS t\n LIMIT 5;"
 
+        return count_tests, query, breakdown
+
+    def _construct_tests(self, columns, table, schema=None):
+        count_tests, query, breakdown = self._construct_tests_template(
+            columns, table, "standard_tests.sql", schema
+        )
         if count_tests == 0:
             return Ok([None, breakdown])
 
@@ -278,6 +289,28 @@ class Database:
 
         return Ok(properties)
 
+    def test_problematic_values_template(
+        self, failed: list, table: str, schema: str, test_file_name: str
+    ) -> str:
+        template = self._jinja_test.get_template(test_file_name)
+        query = ""
+        for f in failed:
+            query += template.render(
+                **{
+                    "table": table,
+                    "schema": schema,
+                    "name": f[2],
+                    "type": f[1],
+                    "allowed_values": f[3],
+                },
+            )
+        return query
+
+    def test_problematic_values(self, failed: list, table: str, schema: str) -> str:
+        return self.test_problematic_values_template(
+            failed, table, schema, "standard_test_output.sql"
+        )
+
     def _refresh_metadata(self, only=None, schema=None):
         """Refreshes the sqlalchemy metadata object.
 
