@@ -1,5 +1,10 @@
+{{ config(
+    materialisation='view'
+   )
+}}
+
 SELECT fr.fighter_name
      , CAST(SUM(fr.is_winner) AS FLOAT) / COUNT(DISTINCT fr.battle_id) AS win_rate
-  FROM f_fighter_results fr
+  FROM {{ src('f_fighter_results') }} fr
  GROUP BY 1
  ORDER BY 2 DESC
