@@ -3,6 +3,8 @@ from enum import Enum
 from pathlib import Path
 from typing import Set, Dict, Any
 
+from colorama import Fore, Style
+
 from ..core.errors import Err, Ok
 from ..logging.task_event_tracker import TaskEventTracker
 from ..utils.compiler import Compiler
@@ -213,16 +215,109 @@ class Task:
         yield
         self._tracker.finish_current_step()
 
-    def get_test_breakdown(self, breakdown):
+    # Test methods
+
+    def get_test_breakdown(self, breakdown: list) -> list:
+        """Return list breakdown of: test status, test type and column.
+
+        Usage:
+        ```python
+        breakdown = self.get_test_breakdown(self.test_breakdown)
+        ```
+
+        Args:
+          breakdown (list): test_breakdown list given by db method _construct_tests
+        """
         data = []
         for brk in breakdown:
             if not brk["execute"]:
-                data.append(["SKIPPED", brk["type"], brk["column"]])
+                data.append(
+                    ["SKIPPED", brk["type"], brk["column"], brk["allowed_values"]]
+                )
             else:
-                data.append(["EXECUTED", brk["type"], brk["column"]])
+                data.append(
+                    ["EXECUTED", brk["type"], brk["column"], brk["allowed_values"]]
+                )
 
         return data
 
+    def test_sucessful(self, breakdown: list) -> Ok:
+        """CLI outputs on successful test execution.
+
+        Args:
+          breakdown (list): output of get_test_breakdown class method.
+        """
+        skipped = [brk for brk in breakdown if brk[0] == "SKIPPED"]
+        executed = [brk for brk in breakdown if brk[0] == "EXECUTED"]
+
+        if skipped:
+            self.info(
+                f"{Fore.GREEN}{len(skipped)} Column test(s) {Style.BRIGHT}SKIPPED{Style.NORMAL}"
+            )
+        self.info(
+            f"{Fore.GREEN}{len(executed)} Column test(s) {Style.BRIGHT}EXECUTED{Style.NORMAL}, {len(executed)} succeeded."
+        )
+
+        return self.success()
+
+    def test_failure(self, breakdown: list, result: dict, run_argument: str) -> tuple:
+        """CLI outputs on failed test execution.
+
+        Args:
+          breakdown (list): output of get_test_breakdown class method.
+          result (dict): output of test query.
+          run_argument (str): "debug" entry in class run_arguments.
+        """
+        skipped = []
+        executed = []
+        failed = []
+        for brk in breakdown:
+            if (sum(brk[1] != res["type"] for res in result) == len(result)) or (
+                sum(brk[2] != res["col"] for res in result) == len(result)
+            ):
+                if brk[0] == "SKIPPED":
+                    skipped.append(brk)
+                if brk[0] == "EXECUTED":
+                    executed.append(brk)
+            else:
+                failed.append(brk)
+        if run_argument:
+
+            fl_info = [f"{Fore.RED}FAILED: "]
+            for info in failed:
+                count = sum(
+                    [
+                        item["cnt"]
+                        for item in result
+                        if (item["type"] == info[1] and item["col"] == info[2])
+                    ]
+                )
+                fl_info.append(
+                    f"{Fore.RED}{Style.BRIGHT}{info[1]} test{Style.NORMAL} on {Style.BRIGHT}{info[2]} FAILED{Style.NORMAL}. {count} offending records."
+                )
+            if skipped:
+                self.info(
+                    f"{Fore.GREEN}{len(skipped)} Column test(s) {Style.BRIGHT}SKIPPED{Style.NORMAL}"
+                )
+            self.info(
+                f"{Fore.GREEN}{len(executed)+len(failed)} Column test(s) {Style.BRIGHT}EXECUTED{Style.NORMAL}, {len(executed)} succeeded."
+            )
+            for err in fl_info:
+                self.info(err)
+
+            errinfo = f"Test Failed. You can find the compiled test query at compile/{self.group}/{self.name}_test.sql. You can find queries to retrieve the problematic values at compile/{self.group}/{self.name}_test_problematic_values.sql"
+
+            return (self.fail(errinfo), failed)
+        else:
+            summary = f"{len(executed)+len(failed)} Column tests were ran, {len(executed)} succeeded, "
+            if skipped:
+                summary += f", {len(skipped)} were skipped, "
+            summary += f"{len(failed)} failed."
+            self.warning(summary)
+            errout = ", ".join(list(set([res["type"] for res in result])))
+
+            return (self.fail(f"Failed test types: {errout}"), failed)
+
     # Jinja methods
 
     def compile_obj(self, obj, **params):
